// global vars

//-- INTERFACE ----------------------
//-- keys
var openSourceKey;
var openTranslationKey;
var saveKey;
var goToNextKey;
	
//-- menuitems
var openSourceMenu;
var openTranslationMenu;
var saveMenu;
var saveAsMenu;
var goToNextMenu;
	
var viewAllMenu;
var viewUntranslatedMenu;
	
//-- buttons
var openSourceBtn;
var saveBtn;
var openTranslationBtn;
var viewAllBtn;
var viewUntranslatedBtn;
var goToNextBtn;

//-- descriptions
var statusbarContent;

//-- textbox
var originalStringBox;
var translationBox;
	
//-- list
var stringCatalog;

//-- charset
// used for source file opening
var sourceCharsetList;
// used for translation file opening and saving of translation
var translationCharsetList;

var sourceCharsetListPopup;
var translationCharsetListPopup;

// End of Interface elements

// array of id, varname, original translation, new translation
var fields = new Array();

// name of the currently selected var
var currentItem = '';

// collection is a 'collection' of items that will be in the view (in the tree)
var collection = null;

//-- FILEPICKER ----------------------
// used for filter of open/save file dialogs
var filterExt = "*.php;*.php2;*.php3;*.php4;*.php5;*.phpx;*.phtml;*.inc;*.phps";
var filterName = "PHP files (" + filterExt + ")";

//-- FILES ----------------------
// path of the source
var sourceFilePath = null;
// name of the source 
var sourceFileName = null;

// path of a "work in progress" file (translation file)
var translationFilePath = null;

// path of the file to save to - saveFile will write into this
var saveFilePath = null; 

//-- STATE ----------------------
// state of application : if true there is a file opened
var translationInProgress = false;
// true if there is new changes since last save
var needSaveFlag = false;
// true if the charset confirmation has not to be shown again
var dontShowAgain = false;

//-- CHARSET ----------------------
// UTF-8 will be use as default if prefs cannot be read
var defaultCharset = 'UTF-8';
var sourceCharsetString;
var translationCharsetString;

//-- DIALOGS ----------------------
// get a reference to the prompt service component.
var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
					.getService(Components.interfaces.nsIPromptService);

/*********************************************************
						STARTUP
 *********************************************************/
/**
 * Function used to init the application at launch
 *
 * @author Piraux Sebastien <pir@cerdecam.be>
 * @desc 	- init all 'element' vars
 *			- handle charset menu
 *          - start a new session of work
 */
function startup()
{
	// object references 
	
	//-- keys
	openSourceKey				= document.getElementById('openSourceKey');
	openTranslationKey			= document.getElementById('openTransKey');
	saveKey						= document.getElementById('saveKey');
	goToNextKey					= document.getElementById('goToNextKey');
	
	//-- menuitems
	openSourceMenu				= document.getElementById('openSourceMenu');
	openTranslationMenu			= document.getElementById('openTransMenu');
	saveMenu					= document.getElementById('saveMenu');
	saveAsMenu					= document.getElementById('saveAsMenu');
	goToNextMenu				= document.getElementById('goToNextMenu');
	
	viewAllMenu					= document.getElementById('viewAllMenu');
	viewUntranslatedMenu		= document.getElementById('viewUntranslatedMenu');
	
	//-- buttons
	goToNextBtn					= document.getElementById('goToNextBtn');
	openSourceBtn				= document.getElementById('openSourceBtn');
	saveBtn 					= document.getElementById('saveBtn');	
	openTranslationBtn 			= document.getElementById('openTransBtn');

	viewAllBtn					= document.getElementById('viewAllBtn');
	viewUntranslatedBtn			= document.getElementById('viewUntranslatedBtn');

	//-- descriptions
    statusbarContent			= document.getElementById('statusbarContent');

	//-- textbox
	originalStringBox			= document.getElementById('originalString-box');
	translationBox				= document.getElementById('translatedString-box');
	
	//-- list
	stringCatalog				= document.getElementById('fieldsList');
	
	//-- charset
	sourceCharsetList			= document.getElementById('sourceCharsetList');
	translationCharsetList		= document.getElementById('translationCharsetList');
	
	sourceCharsetListPopup		= document.getElementById('sourceCharsetListPopup');
	translationCharsetListPopup	= document.getElementById('translationCharsetListPopup');

	//--------
	
	//-- set default charset
	setDefaultCharset();
	
	//-- fill charset lists
	fillCharsetLists();
	
	// start a new translation
	exNewTranslation();
}

/*********************************************************
						QUIT
 *********************************************************/
 
function doQuit()
{
	if( needSaveFlag )
	{
		var quitAction = showConfirmExitDialog();
		
		if( quitAction == 0 ) // Save
		{
			if( saveFile() ) 	return true;		
			else				return false;
		}
		else if( quitAction == 2 ) // don't save
		{
			return true;
		}	
		else //( quitAction == 1 )	 // cancel
		{
			return false;
		}
	}
	else
	{
		return true;
	}
}

function closeWindow()
{
	if( doQuit() )
	{
		window.close();
	}
	return true;
}

/*********************************************************
					NEW TRANSLATION
 *********************************************************/
/**
 * Function used to start a new session
 *
 * @author Piraux Sebastien <pir@cerdecam.be>
 * @desc 	- disable some interface elements
 *          - empty the 2 textzones
 *          - (re)init array of all vars and translations
 *          - empty the list of vars in interface
 */
function exNewTranslation()
{
	// disable some interface elements
	openTranslationDisable(true);
	saveDisable(true);
	goToNextDisable(true);
	viewAllDisable(true);
	viewUntranslatedDisable(true);
	
    // adapt description
	statusbarContent.label = "";
	
	// empty text zones
	originalStringBox.value = "";
	translationBox.value = "";
	
	// will be set the true when the source file will be opened
	translationInProgress = false;
	
	// reset paths
	sourceFilePath = null;
	translationFilePath = null;
	saveFilePath = null;
	
	// (re)init array of all vars and translations
	fields = new Array();

 	// empty the list of vars in interface
	emptyStringCatalog();
}

/**
 * Function used to ask a new session
 *
 * @author Piraux Sebastien <pir@cerdecam.be>
 * @desc display a popup message to ask confirmation if a session is currently
 *       running
 */
function rqNewTranslation()
{
	if ( translationInProgress )
	{

		if( showConfirmNewTranslationDialog() )
		{
			exNewTranslation();
			return true;
		}
		return false;
	}
	else
	{
		exNewTranslation();
		return true;
	}

}

/*********************************************************
				SOURCE FILE HANDLING
 *********************************************************/
function exOpenSourceFile()
{
	if( parseFile(loadFile(sourceFilePath, sourceCharsetString), 'source') )
	{
		fillStringCatalog( 'all' );
		
		// set work in progress flag
		translationInProgress = true;
		
		// enable interface elements
		openTranslationDisable(false);
		saveDisable(false);
		goToNextDisable(false);
		viewAllDisable(false);
		viewUntranslatedDisable(false);
		
		// adapt description
		statusbarContent.label = sourceFileName;
		
		// disable saveFilePath, 
		// to avoid the new translation to be saved on the previously saved file
		saveFilePath = null;
	}
}
 
/**
 * Function used to display the system dialog box used to pick a file
 *
 * @author Piraux Sebastien <pir@cerdecam.be>
 */
function rqOpenSourceFile()
{
	if( rqNewTranslation() )
	{	
		var nsIFilePicker = Components.interfaces.nsIFilePicker;
		var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
		fp.appendFilter(filterName,filterExt);
		fp.init(window, getLocaleString("openReferenceFilepickerTitle"), nsIFilePicker.modeOpen);
		
		var res = fp.show();
		if (res == nsIFilePicker.returnOK)
		{
			sourceFilePath = fp.file.path;
			sourceFileName = fp.file.leafName;
		
			exOpenSourceFile();
		}
	}//else do nothing (cancel)
}

/*********************************************************
				TRANSLATION FILE HANDLING
 *********************************************************/
function exOpenTranslationFile()
{
	if( parseFile(loadFile(translationFilePath, translationCharsetString), 'translation') )
	{
		// refresh translation of selected field 
		useCurrent();
		// disable saveFilePath, 
		// to avoid the new translation to be saved on the previously saved file
		saveFilePath = null;
	}
}

function rqOpenTranslationFile()
{
	var nsIFilePicker = Components.interfaces.nsIFilePicker;
	var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
	fp.appendFilter(filterName,filterExt);
	fp.init(window, getLocaleString("openTranslationFilepickerTitle"), nsIFilePicker.modeOpen);	
	
	var res = fp.show();
	if (res == nsIFilePicker.returnOK)
	{
		translationFilePath = fp.file.path;
		
		exOpenTranslationFile();
	}
}


/*********************************************************
					LOAD A FILE 
 *********************************************************/
function loadFile( filePath, aCharset ) 
{
	try 
	{
		netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
	} 
	catch (e) 
	{
		alert("Permission to read file was denied.");
		return false;
	}
	
	var file = Components.classes["@mozilla.org/file/local;1"]
				.createInstance(Components.interfaces.nsILocalFile);
	file.initWithPath( filePath );

	if ( file.exists() == false ) 
	{
		alert("File does not exist");
		return false;
	}

	// see http://developer.mozilla.org/en/docs/Reading_textual_data
	// read file content
	const replacementChar = Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER;
	
	var fis = Components.classes["@mozilla.org/network/file-input-stream;1"]
				.createInstance( Components.interfaces.nsIFileInputStream );

	fis.init( file, 0x01, 00004, null);	
				
	var is = Components.classes["@mozilla.org/intl/converter-input-stream;1"]
				.createInstance( Components.interfaces.nsIConverterInputStream );
				
	is.init(fis, aCharset, 1024, replacementChar);

	var lis = is.QueryInterface(Components.interfaces.nsIUnicharLineInputStream);

	var data = '';
	var line = {};
	var cont;
		
	do 
	{
		cont = lis.readLine(line);
		data += line.value + "\n";
	}
	while( cont ) 
	
	is.close();
	

	return data;
}


/*********************************************************
							SAVE
 *********************************************************/

function saveFile()
{
	if( !saveFilePath )
	{
		// saveFileAs will call saveFile if successfull
		if( !saveFileAs() )
		{
			return false;
		}
		return true;
	}
	else
	{
		var file = Components.classes["@mozilla.org/file/local;1"]
						.createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath( saveFilePath );
		if ( file.exists() == false ) 
		{
			file.create( Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 420 );
		}
		
		var fos = Components.classes["@mozilla.org/network/file-output-stream;1"]
						.createInstance( Components.interfaces.nsIFileOutputStream );

		fos.init( file, 0x04 | 0x08 | 0x20, 420, 0 );
		
		var cos = Components.classes["@mozilla.org/intl/converter-output-stream;1"]
						.createInstance(Components.interfaces.nsIConverterOutputStream);

		cos.init( fos, translationCharsetString, 0, 0x0000 );

		var output = "<?php\n";
	
		for ( key in fields )
		{
			output += "$" + key + ' = "' + fields[key][1] + '";\n';
		}
		
		output += "?>";
		
		cos.writeString( output );
		cos.flush();
		cos.close();
		
		// change saving state
		needSaveFlag = false;
		
		return true;
	}
}

function saveFileAs()
{
	try
	{
		netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
		netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserWrite");
	}
	catch(e)
	{
		alert("Permission to save file was denied.");
	}

	saveFilePath = null;
	
	var file = rqSaveFile(); // pick the file path
	var path = file.path;  // complete path of the file : path/filename
	var filename = file.leafName; // name of the file

	if( path )
	{
		// test if the filename has an extension
  		var re = /.+\..+$/;
		if( !re.test( filename ) )
		{
			// if the filename doesn't have an extension add '.php'
			path = path + '.php';
		}
		saveFilePath = path;

  		saveFile();

		return true;
	}
	return false;
	
}

function rqSaveFile()
{
	var nsIFilePicker = Components.interfaces.nsIFilePicker;
	var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
	fp.appendFilter(filterName,filterExt);
	fp.init(window, getLocaleString("saveAsFilepickerTitle"), nsIFilePicker.modeSave);	
	
	var res = fp.show();
	if (res == nsIFilePicker.returnOK || res == nsIFilePicker.returnReplace )
	{
		return fp.file;
	}
	else
	{
		return false;
	}
}

/*********************************************************
				PARSE PHP FILE
 *********************************************************/
function parseFile( fileContent, type )
{
	if( type == 'source' ) 
		var typeIsSource = true;
	else
		var typeIsSource = false;
		
	//-- file definition
	var commentsDelimiters = new Array();
	commentsDelimiters['//'] 	= "\n";
	commentsDelimiters['#'] 	= "\n";
	commentsDelimiters['/*']	= "*/";	

	var escapeChar = "\\";

    // used when in a a var but not between some [ ]
	var re = /[\w]/;  // \w == [a-zA-Z0-9_]
	// used when between [ and ] in an array var because there we may have a space or quotes  or dot (.) or hyphen (-)
	var reInArray = /[^\[\]]/;
	
	var contentLength = fileContent.length;

	var thisChar;
	
	var openString = ''; // quotemark use to open last string we met
	var inVar = false; // true if we are in a normal var $varname, or in an array var after a ]
	var inArrayVar = false; // true if we are in a array var $array['varname'], when between [ and ]
	var lastOpenPos = 0; // position of the last string/variable opening (first char right after the quotemark)
	
	var lastVar; // keep the name of the last var
	
	var isEscaped = false;
	
	for( var i = 0; i < contentLength; i++ )
	{
		thisChar = fileContent.substr(i, 1);
		
		if( inVar )
		{
			// if we detect a [ in the varname it means this is an array var
			if( thisChar =='[' )
			{
				inVar = false;
				inArrayVar = true;
			}
			else if( !re.test(thisChar) )
			{
				lastVar = fileContent.substr( lastOpenPos, i - lastOpenPos );
				inVar = false;
			}
		}
		else if( inArrayVar )
		{
			// when we are in a array type var, between the [ ] we can have some other chars like space, quotes, dot, hyphens
			// so the regex has to be different
			// if we meet a ] go back to normal use of
    		if( thisChar == ']' )
			{
				inVar = true;
				inArrayVar = false;
			}
			else if( !reInArray.test(thisChar) )
			{
				lastVar = fileContent.substr( lastOpenPos, i - lastOpenPos );
				inVar = false;
				inArrayVar = false;
			}
		}
		else if( isEscaped )
		{
			// thisChar is escaped
			isEscaped = false;
		}
		// if we match the escape char
		else if( thisChar == escapeChar )
		{
			// set the escape flag to escape the next char
			isEscaped = true;
		}
		// if we match the end of a string
		else if( thisChar == openString )
		{
			// end of the last string
			//alert(lastOpenPos + " " + (i - lastOpenPos) + " " + fileContent.substr( lastOpenPos, i - lastOpenPos ));
			
			if( lastVar != '' && isDefined(lastVar) )
			{
				if( typeIsSource )
				{
					if( typeof(fields[lastVar]) == 'undefined' )
					{					
						fields[lastVar] = new Array();
						fields[lastVar][1] = ''; // translation
					}
					fields[lastVar][0] = fileContent.substr( lastOpenPos, i - lastOpenPos ); //string to translate						
				}
				else
				{
					if( typeof(fields[lastVar]) != 'undefined' )
					{
						fields[lastVar][1] = fileContent.substr( lastOpenPos, i - lastOpenPos );
					}
				}
			}
			openString = '';
		}
		// if we are not in a string
		else if( openString == '' ) 
		{
			// check for string delimiters
			if( thisChar == '"' || thisChar == '\'' )
			{
				// beginnning of a string
				openString = thisChar;	
				lastOpenPos = i + 1;
			}
			// check for a variable delimiter
			else if( thisChar == '$' )
			{
				inVar = true;
				lastOpenPos = i + 1;			
				lastVar = '';
			}
			// check for a comment
			else if( thisChar == '#' || thisChar == '/' )
			{
				// check if we match the start of a comment
				for ( openTag in commentsDelimiters )
				{
					// take the next chars according to the length of the opening tag
					var testStr = fileContent.substr( i , openTag.length);
					
					if( testStr == openTag )
					{
						// find position of closing tag
						//var restOfContent = fileContent.substr(i, contentLength - i);
						//var closePos = restOfContent.indexOf(commentsDelimiters[openTag]);
						var closePos = fileContent.indexOf(commentsDelimiters[openTag], i);

						var openPos = i;
						// increment iterator to continue parsing from the end of the comment line
						i = closePos + commentsDelimiters[openTag].length - 1;
						//alert("-" + fileContent.substr(openPos-5,10) + "." + "\ncomment : \n" + fileContent.substr(openPos,i - openPos) + "\n" + fileContent.substr(i,1) + "\nopenPos : " + openPos + "\ni - openPos : " + (i - openPos) + "\nclosePos : " + closePos +  "\ncommentsDelimiters[openTag].length : " + commentsDelimiters[openTag].length);
						break; // exist from the opentag loop, we have found what we were looking for
					}
				}
			} // else do nothing, it's a 'we don't care'-char
		}
	}
	return true;
}


/*********************************************************
						MODEL
 *********************************************************/
function treeModel()
{
  	this.rowCount = collection.size();

	this.getCellText = function(row,col)
	{
  		return collection.get(row);
 	}
	this.setTree = function(treebox){ this.treebox = treebox; }
	this.isContainer = function(row){ return false; }
	this.isSeparator = function(row){ return false; }
	this.isSorted = function(row){ return false; }
	this.getLevel = function(row){ return 0; }
	this.getImageSrc = function(row,col){ return null;}
	this.getRowProperties = function(row,props) {}
	this.getCellProperties = function(row,col,props){}
	this.getColumnProperties = function(colid,col,props) {}
}
/*********************************************************
				COLLECTIONS
 *********************************************************/
function completeCollection()
{
	var items = new Array();

	for ( key in fields )
	{
        // display all vars
		items.push(key);
	}

	this.get  = function(index) { return items[index]; }
	this.size = function() { return items.length; }
	this.getArray = function() { return items; }
}

function untranslatedCollection()
{
	var items = new Array();

	for ( key in fields )
	{
        // display only vars without translation
		if ( fields[key][1] == "" ) items.push(key);
	}

	this.get  = function(index) { return items[index]; }
	this.size = function() { return items.length; }
	this.getArray = function() { return items; }
}

function emptyCollection()
{
	var items = new Array();

	// empty collection so ... the items remains empty

	this.get  = function(index) { return items[index]; }
	this.size = function() { return items.length; }
	this.getArray = function() { return items; }
}


/*********************************************************
						OTHERS
 *********************************************************/
 
function useCurrent()
{
	// value of selected element in view
    currentItem = collection.get(stringCatalog.currentIndex);

	if( isDefined(fields[currentItem][0]) )
	{
		originalStringBox.value = fields[currentItem][0];
	}
	else
	{
		originalStringBox.value = "";
	}

	if( isDefined(fields[currentItem][1]) )
	{
		translationBox.value = fields[currentItem][1];
	}
	else
	{
		translationBox.value = "";
	}
}

function saveCurrentItem()
{
	// handle the double quotes problem
	// replace possible \" by " and then replace all " by \"
	var translated = translationBox.value.replace(/\\\"/g, '"');
	translated = translated.replace(/\"/g, '\\"');
	
	// memorise new translation
	fields[currentItem][1] = translated;
	
	// change saving state
	needSaveFlag = true;
}

function goToNext()
{
    // get index of next element, or 0 if the current item is the last one
	if ( stringCatalog.currentIndex < collection.size()-1 )
		var nextItemIndex = stringCatalog.currentIndex + 1;
	else
		var nextItemIndex = 0;

	// select next item
	stringCatalog.view.selection.select( nextItemIndex );
	// ensure the selected row will be displayed
	stringCatalog.treeBoxObject.ensureRowIsVisible( nextItemIndex );
	// select all the text in the translation textbox
	translationBox.select();
	// give focus back to the translation textbox
	translationBox.focus();	
}

function fillStringCatalog( mode )
{
	// empty listbox to avoid duplicate entries
	emptyStringCatalog();

	// empty text zones
	originalStringBox.value = "";
	translationBox.value = "";

	if ( mode == 'empty' )
		collection = new untranslatedCollection();
	else
		collection = new completeCollection();

	// refresh the view
	stringCatalog.view = new treeModel();

	// select first item 
	stringCatalog.view.selection.select( 0 );
}


function emptyStringCatalog()
{
	// reset the collection, so the view will be empty
	collection = new emptyCollection();

 	stringCatalog.view = new treeModel();
}

function isDefined(a) 
{ 
	return typeof a != 'undefined';
}

function isNull(a) 
{
    return typeof a == 'object' && !a;
}

/*********************************************************
	ENABLE/DISABLE FUNCTION FOR MAIN INTERFACE ELEMENTS

	these functions are a convenient way to activate
	or deactivate interface elements
 *********************************************************/

function openSourceDisable( disable )
{
	openSourceKey.disabled = disable;
	openSourceMenu.setAttribute('disabled',disable);
	openSourceBtn.disabled = disable;
}

function openTranslationDisable( disable )
{
	openTranslationKey.disabled = disable;
	openTranslationMenu.setAttribute('disabled',disable);
	openTranslationBtn.disabled = disable;
}

function saveDisable( disable )
{
	saveKey.disabled = disable;
	saveMenu.setAttribute('disabled',disable);
	saveAsMenu.setAttribute('disabled',disable);
	saveBtn.disabled = disable;
}

function viewAllDisable( disable )
{	
	viewAllMenu.setAttribute('disabled',disable);
	viewAllBtn.disabled = disable;
}

function viewUntranslatedDisable( disable )
{
	viewUntranslatedMenu.setAttribute('disabled',disable);
	viewUntranslatedBtn.disabled = disable;
}

function goToNextDisable( disable )
{
	goToNextKey.disabled = disable;
	goToNextMenu.setAttribute('disabled',disable);
	goToNextBtn.disabled = disable;
}

/*********************************************************
				DIALOGS WITH USER
 *********************************************************/
function showConfirmNewTranslationDialog()
{
	// checkResult will store the state of the check box
	var checkResult = {};
	
	// show a confirm dialog with a checkbox
	return promptService.confirm(window,getLocaleString("newTranslationDialogTitle"),
	  getLocaleString("newTranslationDialog"));
}

function showConfirmExitDialog()
{
	// set the buttons that will appear on the dialog
	var flags=promptService.BUTTON_TITLE_SAVE * promptService.BUTTON_POS_0 +
			  promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_1 +
			  promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_2;
	
	// display the dialog
	return promptService.confirmEx(window,getLocaleString("saveTranslationDialogTitle"),
		  getLocaleString("saveTranslationDialog"),
		  flags, null, null, getLocaleString("saveTranslationDialogDontSaveButton"), null, {});
}


function showConfirmChangeCharsetDialog()
{
	if( !dontShowAgain )
	{
		// checkResult will store the state of the check box
		var checkResult = {};
		
		// show a confirm dialog with a checkbox
		var returnValue = promptService.confirmCheck(window,getLocaleString("modifyCharsetDialogTitle"),
		  getLocaleString("modifyCharsetDialog"),
		  getLocaleString("modifyCharsetDialogCheckBox"),
		  checkResult);
		
		// if the check box was checked keep the state to not display it next time
		if (checkResult.value) dontShowAgain = true;
		
		return returnValue;
	}
	else
	{
		return true;
	}
}

/*********************************************************
						CHARSET
 *********************************************************/
 
function setDefaultCharset()
{
	/*
	// REMOVED AS IT SEEMS TO RETURN AN INCORRECT VALUE IN SOME CASES ...	
	// view https://bugzilla.mozilla.org/show_bug.cgi?id=163271
	try
	{
		var prefs = Components.classes["@mozilla.org/preferences-service;1"]
						.getService(Components.interfaces.nsIPrefBranch);
						
		defaultCharset = prefs.getCharPref("intl.charset.default");
	}
	catch (ex) {}
	*/
	
	sourceCharsetString = defaultCharset;
	translationCharsetString = defaultCharset;
}

var sourceCharsetListListener = {
	item: null,
	willRebuild : function(builder) {
		// do nothing
	},
	didRebuild : function(builder) {
    	sourceCharsetList.value = sourceCharsetString;
	}
};

var translationCharsetListListener = {
	item: null,
	willRebuild : function(builder) {
		// do nothing
	},
	didRebuild : function(builder) {
    	translationCharsetList.value = translationCharsetString;
  	}
};

function fillCharsetLists()
{	
	var RDF = Components.classes["@mozilla.org/rdf/rdf-service;1"].
            getService(Components.interfaces.nsIRDFService);
	
	var ds = RDF.GetDataSourceBlocking("chrome://phplangeditor/content/charsets.rdf");
	
	sourceCharsetListPopup.builder.addListener(sourceCharsetListListener);
	sourceCharsetListPopup.database.AddDataSource(ds);
	sourceCharsetListPopup.builder.rebuild();
	
	translationCharsetListPopup.builder.addListener(translationCharsetListListener);
	translationCharsetListPopup.database.AddDataSource(ds);
	translationCharsetListPopup.builder.rebuild();
}


function setSourceCharset()
{	
	if( !translationInProgress || showConfirmChangeCharsetDialog())
	{
		sourceCharsetString = sourceCharsetList.value;
		// reopen the file to read it with the choosen charset
		if(sourceFilePath)	exOpenSourceFile();
	}
	else
	{
		// user has cancelled charset change so display the previously selected charset
		sourceCharsetList.value = sourceCharsetString;
		
	}
}
 
function setTranslationCharset()
{
	translationCharsetString = translationCharsetList.value;
	if(translationFilePath) exOpenTranslationFile();
}

function getLocaleString(strName)
{
	var strbundle = document.getElementById("phplangeditor-strings");
	try
	{
		return strbundle.getString(strName);
	}
	catch( e )
	{
	 	return strName;
	}
}